#version 300 es
precision lowp float;

layout(location = 0) out vec4 output_color;
layout(location = 1) out ivec4 pos_in_world;

in vec3 frag_color;
in vec3 frag_position;
in vec3 frag_normal;
in vec3 frag_pos_in_world;
in vec4 frag_pos_in_light_space;

uniform float ambient_strength;
uniform vec3 light_color;
uniform vec3 light_direction;
uniform lowp sampler2DArray decals;
uniform lowp isampler2D decal_indices;
uniform lowp sampler2D shadowmap;

float shadow(vec4 frag_pos_in_light_space) {
    vec3 proj_coords = frag_pos_in_light_space.xyz / frag_pos_in_light_space.w * 0.5 + 0.5;
//    float closest_depth = texture(shadowmap, proj_coords.xy).r;
    float current_depth = proj_coords.z;
//    return current_depth - bias > closest_depth ? 1.0 : 0.0;

    float shadow = 0.0;
    vec2 texel_size = 1.0 / vec2(textureSize(shadowmap, 0));
    float bias = max(0.005 * (1.0 - dot(frag_normal, -light_direction)), texel_size.x);
    //float bias = max(0.5 * (1.0 - dot(frag_normal, -light_direction)), 0.005);
    //float bias = 0.0;
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcf_depth = texture(shadowmap, proj_coords.xy + vec2(x, y) * texel_size).r;
            shadow += current_depth - bias > pcf_depth ? 1.0 : 0.0;
        }
    }
    shadow /= 9.0;
    return shadow;
}

void main() {
    vec3 ambient_color = ambient_strength * light_color;
    vec3 diffuse_color = max(dot(frag_normal, -light_direction), 0.0) * light_color;
    vec2 world_size = vec2(textureSize(decal_indices, 0));
    //vec2 world_size = vec2(32, 32);
    int decal_index = texelFetch(decal_indices, ivec2(frag_pos_in_world.xy * world_size), 0).r;
    //int decal_index = 0;
//    vec3 tex_coord = vec3(frag_pos_in_world.xy * vec2(textureSize(decal_indices, 0)), decal_index);
    vec3 tex_coord = vec3(frag_pos_in_world.xy * world_size, decal_index);
    vec4 tex_color = texture(decals, tex_coord);
    vec3 obj_color = mix(frag_color, tex_color.rgb, tex_color.a);
    vec3 color = (ambient_color + (1.0 - shadow(frag_pos_in_light_space)) * diffuse_color) * obj_color;
    output_color = vec4(color, 1.0);
//  output_color = vec4(1.0, 1.0, 1.0, 1.0);
    pos_in_world = ivec4(frag_pos_in_world.xy * 1024.0, world_size.x, 1);
//  tile = vec2(0.5, 0.3);
}
